package main

import (
	"go.uber.org/zap"
	"miniapp/client"
	"miniapp/oauth2"
	"miniapp/task"

	"miniapp/core"
	"miniapp/global"
	"miniapp/initialize"
)

//go:generate go env -w GO111MODULE=on
//go:generate go env -w GOPROXY=https://goproxy.cn,direct
//go:generate go mod tidy
//go:generate go mod download

// @title                       Swagger Example API
// @version                     0.0.1
// @description                 This is a sample Server pets
// @securityDefinitions.apikey  ApiKeyAuth
// @in                          header
// @name                        x-token
// @BasePath                    /

func main() {
	global.GVA_VP = core.Viper() // 初始化Viper
	initialize.OtherInit()
	global.GVA_LOG = core.Zap() // 初始化zap日志库
	zap.ReplaceGlobals(global.GVA_LOG)
	global.GVA_DB = initialize.Gorm() // gorm连接数据库
	initialize.Timer()
	initialize.DBList()
	if global.GVA_DB != nil {
		initialize.RegisterTables() // 初始化表
		// 程序结束前关闭数据库链接
		db, _ := global.GVA_DB.DB()
		defer db.Close()
	}

	client.InitRedisClient()
	oauth2.InitOAuth2Server() // 初始化OAuth2服务
	initialize.InitKp()       // 初始化关键词处理器
	task.InitTask()           // 初始化定时任务
	core.RunWindowsServer()
}
